@@ -5,13 +5,14 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
-#include <KIO/JobTracker>
-#include <KJobTrackerInterface>
-#include <KJob>
 #include <KFile>
+#include <KIO/JobTracker>
+#include <KJob>
+#include <KJobTrackerInterface>
 
-#include <QtTest>
 #include <QEventLoop>
+#include <QTest>
+#include <QTimer>
 
 // widget is shown with hardcoded delay of 500 ms by KWidgetJobTracker
 static const int testJobRunningTime = 600;
@@ -20,13 +21,18 @@ class TestJob : public KJob
 {
     Q_OBJECT
 public:
-    void start() override { QTimer::singleShot(testJobRunningTime, this, &TestJob::doEmit); }
+    void start() override
+    {
+        QTimer::singleShot(testJobRunningTime, this, &TestJob::doEmit);
+    }
 
 private Q_SLOTS:
-    void doEmit() { emitResult(); }
+    void doEmit()
+    {
+        emitResult();
+    }
 };
 
-
 class KDynamicJobTrackerTest : public QObject
 {
     Q_OBJECT
@@ -41,7 +47,7 @@ void KDynamicJobTrackerTest::testNoCrashWithoutQWidgetsPossible()
     KFile::isDefaultView(KFile::Default);
 
     // simply linking to KIOWidgets results in KDynamicJobTracker installing itself as KIO's jobtracker
-    KJobTrackerInterface* jobtracker = KIO::getJobTracker();
+    KJobTrackerInterface *jobtracker = KIO::getJobTracker();
     QCOMPARE(jobtracker->metaObject()->className(), "KDynamicJobTracker");
 
     TestJob *job = new TestJob;
