@@ -1004,6 +1004,11 @@ int KDirOperatorPrivate::sortColumn() const
 
 Qt::SortOrder KDirOperatorPrivate::sortOrder() const
 {
+    // Reverse whatever was applied to time sorting, since reversed by default is nicer
+    if (m_sorting & QDir::Time) {
+        return (m_sorting & QDir::Reversed) ? Qt::AscendingOrder : Qt::DescendingOrder;
+    }
+
     return (m_sorting & QDir::Reversed) ? Qt::DescendingOrder : Qt::AscendingOrder;
 }
 
@@ -1020,20 +1025,6 @@ void KDirOperatorPrivate::updateSorting(QDir::SortFlags sort)
     m_proxyModel->setSortFoldersFirst(m_sorting & QDir::DirsFirst);
     m_proxyModel->sort(sortColumn(), sortOrder());
 
-    // TODO: The headers from QTreeView don't take care about a sorting
-    // change of the proxy model hence they must be updated the manually.
-    // This is done here by a qobject_cast, but it would be nicer to:
-    // - provide a signal 'sortingChanged()'
-    // - connect KDirOperatorDetailView() with this signal and update the
-    //   header internally
-    QTreeView *treeView = qobject_cast<QTreeView *>(m_itemView);
-    if (treeView != nullptr) {
-        QHeaderView *headerView = treeView->header();
-        headerView->blockSignals(true);
-        headerView->setSortIndicator(sortColumn(), sortOrder());
-        headerView->blockSignals(false);
-    }
-
     assureVisibleSelection();
 }
 
@@ -2362,8 +2353,8 @@ void KDirOperator::updateSortActions()
         ascending->setText(i18nc("Sort ascending", "A-Z"));
     } else if (KFile::isSortByDate(d->m_sorting)) {
         action(KDirOperator::SortByDate)->setChecked(true);
-        descending->setText(i18nc("Sort descending", "Newest First"));
-        ascending->setText(i18nc("Sort ascending", "Oldest First"));
+        descending->setText(i18nc("Sort ascending", "Oldest First"));
+        ascending->setText(i18nc("Sort descending", "Newest First"));
     } else if (KFile::isSortBySize(d->m_sorting)) {
         action(KDirOperator::SortBySize)->setChecked(true);
         descending->setText(i18nc("Sort descending", "Largest First"));
