@@ -88,6 +88,7 @@ static void modifyEnv(KProcess &process, QProcessEnvironment mod)
 KProcessRunner *KProcessRunner::fromApplication(const KService::Ptr &service,
                                                 const QString &serviceEntryPath,
                                                 const QList<QUrl> &urls,
+                                                const QString &actionName,
                                                 KIO::ApplicationLauncherJob::RunFlags flags,
                                                 const QString &suggestedFileName,
                                                 const QByteArray &asn)
@@ -103,9 +104,12 @@ KProcessRunner *KProcessRunner::fromApplication(const KService::Ptr &service,
     const bool notYetSupportedOpenActivationNeeded = !urls.isEmpty();
     if (!notYetSupportedOpenActivationNeeded && DBusActivationRunner::activationPossible(service, flags, suggestedFileName)) {
         const auto actions = service->actions();
-        auto action = std::find_if(actions.cbegin(), actions.cend(), [service](const KServiceAction &action) {
-            return action.exec() == service->exec();
+        auto action = std::find_if(actions.cbegin(), actions.cend(), [actionName](const KServiceAction &action) {
+            return action.name() == actionName;
         });
+        if (!actionName.isEmpty() && action == actions.cend()) {
+            qCWarning(KIO_GUI) << "Requested action" << actionName << "cannot be found for" << service->name();
+        }
         instance = new DBusActivationRunner(action != actions.cend() ? action->name() : QString());
     } else {
         instance = makeInstance();
@@ -140,7 +144,8 @@ KProcessRunner *KProcessRunner::fromApplication(const KService::Ptr &service,
 
     QString workingDir(service->workingDirectory());
     if (workingDir.isEmpty() && !urls.isEmpty() && urls.first().isLocalFile()) {
-        workingDir = urls.first().adjusted(QUrl::RemoveFilename).toLocalFile();
+        // systemd requires working directory to be normalized, or '~'
+        workingDir = QFileInfo(urls.first().toLocalFile()).canonicalPath();
     }
     instance->m_process->setWorkingDirectory(workingDir);
 
@@ -192,7 +197,7 @@ KProcessRunner *KProcessRunner::fromExecutable(const QString &executable,
 {
     const QString actualExec = QStandardPaths::findExecutable(executable);
     if (actualExec.isEmpty()) {
-        qCWarning(KIO_GUI) << "Could not find an executable named:" << executable;
+        qCInfo(KIO_GUI) << "Could not find an executable named:" << executable;
         return {};
     }
 
@@ -304,31 +309,26 @@ void KProcessRunner::init(const KService::Ptr &service, const QString &serviceEn
         if (!asn.isEmpty()) {
             m_process->setEnv(QStringLiteral("XDG_ACTIVATION_TOKEN"), QString::fromUtf8(asn));
         } else {
-            bool silent;
-            QByteArray wmclass;
-            const bool startup_notify = service && KIOGuiPrivate::checkStartupNotify(service.data(), &silent, &wmclass);
-            if (startup_notify && !silent) {
-                auto window = qGuiApp->focusWindow();
-                if (!window && !qGuiApp->allWindows().isEmpty()) {
-                    window = qGuiApp->allWindows().constFirst();
-                }
-                if (window) {
-                    const int launchedSerial = KWaylandExtras::lastInputSerial(window);
-                    m_waitingForXdgToken = true;
-                    connect(
-                        KWaylandExtras::self(),
-                        &KWaylandExtras::xdgActivationTokenArrived,
-                        m_process.get(),
-                        [this, launchedSerial](int tokenSerial, const QString &token) {
-                            if (tokenSerial == launchedSerial) {
-                                m_process->setEnv(QStringLiteral("XDG_ACTIVATION_TOKEN"), token);
-                                m_waitingForXdgToken = false;
-                                startProcess();
-                            }
-                        },
-                        Qt::SingleShotConnection);
-                    KWaylandExtras::requestXdgActivationToken(window, launchedSerial, resolveServiceAlias());
-                }
+            auto window = qGuiApp->focusWindow();
+            if (!window && !qGuiApp->allWindows().isEmpty()) {
+                window = qGuiApp->allWindows().constFirst();
+            }
+            if (window) {
+                const int launchedSerial = KWaylandExtras::lastInputSerial(window);
+                m_waitingForXdgToken = true;
+                connect(
+                    KWaylandExtras::self(),
+                    &KWaylandExtras::xdgActivationTokenArrived,
+                    m_process.get(),
+                    [this, launchedSerial](int tokenSerial, const QString &token) {
+                        if (tokenSerial == launchedSerial) {
+                            m_process->setEnv(QStringLiteral("XDG_ACTIVATION_TOKEN"), token);
+                            m_waitingForXdgToken = false;
+                            startProcess();
+                        }
+                    },
+                    Qt::SingleShotConnection);
+                KWaylandExtras::requestXdgActivationToken(window, launchedSerial, resolveServiceAlias());
             }
         }
     }
@@ -422,7 +422,7 @@ QString KProcessRunner::escapeUnitName(const QString &input)
 {
     QString res;
     const QByteArray bytes = input.toUtf8();
-    for (const auto &c : bytes) {
+    for (const unsigned char c : bytes) {
         if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c >= '0' && c <= '9') || c == ':' || c == '_' || c == '.') {
             res += QLatin1Char(c);
         } else {
@@ -445,7 +445,8 @@ QString KProcessRunner::resolveServiceAlias() const
 
 void KProcessRunner::emitDelayedError(const QString &errorMsg)
 {
-    qCWarning(KIO_GUI) << errorMsg;
+    qCWarning(KIO_GUI) << name() << errorMsg;
+
     terminateStartupNotification();
     // Use delayed invocation so the caller has time to connect to the signal
     auto func = [this, errorMsg]() {
@@ -458,15 +459,16 @@ void KProcessRunner::emitDelayedError(const QString &errorMsg)
 void ForkingProcessRunner::slotProcessExited(int exitCode, QProcess::ExitStatus exitStatus)
 {
     qCDebug(KIO_GUI) << name() << "exitCode=" << exitCode << "exitStatus=" << exitStatus;
-    terminateStartupNotification();
-    deleteLater();
 #ifdef Q_OS_UNIX
     if (exitCode == 127) {
 #else
     if (exitCode == 9009) {
 #endif
         const QStringList args = m_cmd.split(QLatin1Char(' '));
-        emitDelayedError(xi18nc("@info", "The command <command>%1</command> could not be found.", args[0]));
+        emitDelayedError(xi18nc("@info", "The command <command>%1</command> could not be found.", args[0])); // Calls deleteLater().
+    } else {
+        terminateStartupNotification();
+        deleteLater();
     }
 }
 
