@@ -33,6 +33,10 @@
 #include <authinfo.h>
 #include <ksslcertificatemanager.h>
 
+#ifndef Q_OS_WIN
+#include <sys/utsname.h>
+#endif
+
 // Pseudo plugin class to embed meta data
 class KIOPluginForMetaData : public QObject
 {
@@ -94,6 +98,27 @@ public:
     }
 };
 
+namespace
+{
+
+bool isDav(const QString &protocol)
+{
+    return protocol.startsWith(QLatin1String("webdav")) || protocol.startsWith(QLatin1String("dav"));
+}
+
+QUrl protocolChangedToHttp(const QUrl &url)
+{
+    QUrl newUrl{url};
+    QString protocol{newUrl.scheme()};
+    // Keeps the 's' at the end, if present.
+    protocol.replace(QLatin1String{"webdav"}, QLatin1String{"http"});
+    protocol.replace(QLatin1String{"dav"}, QLatin1String{"http"});
+    if (newUrl.scheme() != protocol)
+        newUrl.setScheme(protocol);
+    return newUrl;
+}
+};
+
 HTTPProtocol::HTTPProtocol(const QByteArray &protocol, const QByteArray &pool, const QByteArray &app)
     : WorkerBase(protocol, pool, app)
 {
@@ -233,8 +258,17 @@ HTTPProtocol::Response HTTPProtocol::makeDavRequest(const QUrl &url,
 HTTPProtocol::Response
 HTTPProtocol::makeRequest(const QUrl &url, KIO::HTTP_METHOD method, QByteArray &inputData, DataMode dataMode, const QMap<QByteArray, QByteArray> &extraHeaders)
 {
+    /* HTTPProtocol::get(...) creates an empty inputData whether or not the calling function
+     * sent data to the request. QNetworkRequest sends "Content-Length: 0" for all requests
+     * when the device != nullptr, even if data is empty. Per RFC9110, "A user agent SHOULD NOT
+     * send a Content-Length header field when the request message does not contain content and
+     * the method semantics do not anticipate such data." Semantically, HTTP_GET, HTTP_HEAD,
+     * and others shouldn't send the header when the data is empty. Workaround that behavior
+     * here until and if Qt is modified. https://bugreports.qt.io/browse/QTBUG-138848 */
+    const bool noBodyWhenEmpty = (method == KIO::HTTP_GET || method == KIO::HTTP_HEAD || method == KIO::HTTP_DELETE);
     QBuffer buffer(&inputData);
-    return makeRequest(url, method, &buffer, dataMode, extraHeaders);
+    QIODevice *bodyDevice = (noBodyWhenEmpty && inputData.isEmpty()) ? nullptr : &buffer;
+    return makeRequest(url, method, bodyDevice, dataMode, extraHeaders);
 }
 
 static QString protocolForProxyType(QNetworkProxy::ProxyType type)
@@ -279,13 +313,7 @@ HTTPProtocol::Response HTTPProtocol::makeRequest(const QUrl &url,
 
     nam.setCookieJar(cookies);
 
-    QUrl properUrl = url;
-    if (url.scheme() == QLatin1String("webdav")) {
-        properUrl.setScheme(QStringLiteral("http"));
-    }
-    if (url.scheme() == QLatin1String("webdavs")) {
-        properUrl.setScheme(QStringLiteral("https"));
-    }
+    QUrl properUrl = protocolChangedToHttp(url);
 
     m_hostName = properUrl.host();
 
@@ -398,10 +426,11 @@ HTTPProtocol::Response HTTPProtocol::makeRequest(const QUrl &url,
         request.setRawHeader("Referer" /* sic! */, referrer.toUtf8());
     }
 
-    const QString userAgent = metaData(QStringLiteral("UserAgent"));
-    if (!userAgent.isEmpty()) {
-        request.setHeader(QNetworkRequest::UserAgentHeader, userAgent.toUtf8());
+    QString userAgent = metaData(QStringLiteral("UserAgent"));
+    if (userAgent.isEmpty()) {
+        userAgent = defaultUserAgent();
     }
+    request.setHeader(QNetworkRequest::UserAgentHeader, userAgent.toUtf8());
 
     const QString accept = metaData(QStringLiteral("accept"));
     if (!accept.isEmpty()) {
@@ -428,6 +457,10 @@ HTTPProtocol::Response HTTPProtocol::makeRequest(const QUrl &url,
         }
     }
 
+    if (inputData) {
+        inputData->startTransaction(); // To be able to restart after redirects.
+    }
+
     QNetworkReply *reply = nam.sendCustomRequest(request, methodToString(method), inputData);
 
     bool mimeTypeEmitted = false;
@@ -449,12 +482,47 @@ HTTPProtocol::Response HTTPProtocol::makeRequest(const QUrl &url,
         processedSize(received);
     });
 
-    QObject::connect(reply, &QNetworkReply::metaDataChanged, [this, &mimeTypeEmitted, reply, dataMode, url, method]() {
-        handleRedirection(method, url, reply);
+    // From RFC 4918 5.2 Collection Resources:
+    // > In general, clients SHOULD use the trailing slash form of collection names.
+    // > If clients do not use the trailing slash form the client needs to be prepared to see a redirect response.
+    // KIO doesn't handle trailing slashes well (especially in KDirLister), so handle it transparently.
+    bool redirectToTrailingSlash = false;
 
-        int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    QObject::connect(reply, &QNetworkReply::metaDataChanged, [this, &mimeTypeEmitted, &redirectToTrailingSlash, reply, dataMode, url, method]() {
+        const int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
 
-        if (statusCode == 206) {
+        if (statusCode >= 300 && statusCode < 400) {
+            const QString redir = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toString();
+            const QUrl newUrl = url.resolved(QUrl(redir));
+
+            // Handled after returning from the event loop.
+            // 301 is necessary for Apache (see bugs 209508 and 187970).
+            if (statusCode == 301 || statusCode == 307 || statusCode == 308) {
+                if (url != newUrl && url == newUrl.adjusted(QUrl::StripTrailingSlash)) {
+                    redirectToTrailingSlash = true;
+                    return;
+                }
+            }
+
+            if (statusCode == 301 || statusCode == 308) {
+                setMetaData(QStringLiteral("permanent-redirect"), QStringLiteral("true"));
+                redirection(newUrl);
+            } else if (statusCode == 302) {
+                if (method == KIO::HTTP_POST) {
+                    setMetaData(QStringLiteral("redirect-to-get"), QStringLiteral("true"));
+                }
+
+                redirection(newUrl);
+            } else if (statusCode == 303) {
+                if (method != KIO::HTTP_HEAD) {
+                    setMetaData(QStringLiteral("redirect-to-get"), QStringLiteral("true"));
+                }
+
+                redirection(newUrl);
+            } else if (statusCode == 307) {
+                redirection(newUrl);
+            }
+        } else if (statusCode == 206) {
             canResume();
         }
 
@@ -492,6 +560,19 @@ HTTPProtocol::Response HTTPProtocol::makeRequest(const QUrl &url,
     });
     loop.exec();
 
+    // If there was a foo -> foo/ redirect, follow it.
+    if (redirectToTrailingSlash) {
+        if (inputData) {
+            inputData->rollbackTransaction();
+        }
+        QUrl newUrl = url;
+        newUrl.setPath(newUrl.path() + QLatin1Char('/'));
+        return makeRequest(newUrl, method, inputData, dataMode, extraHeaders);
+    }
+    if (inputData) {
+        inputData->commitTransaction();
+    }
+
     // make sure data is emitted at least once
     // NOTE: emitting an empty data set means "end of data" and must not happen
     // before we have set up our metadata properties etc. Only emit this at the
@@ -556,7 +637,7 @@ KIO::WorkerResult HTTPProtocol::get(const QUrl &url)
 
 KIO::WorkerResult HTTPProtocol::put(const QUrl &url, int /*_mode*/, KIO::JobFlags flags)
 {
-    if (url.scheme().startsWith(QLatin1String("webdav"))) {
+    if (isDav(url.scheme())) {
         if (!(flags & KIO::Overwrite)) {
             // Checks if the destination exists and return an error if it does.
             if (davDestinationExists(url)) {
@@ -643,38 +724,6 @@ QString HTTPProtocol::getContentType()
     return contentType;
 }
 
-void HTTPProtocol::handleRedirection(KIO::HTTP_METHOD method, const QUrl &originalUrl, QNetworkReply *reply)
-{
-    int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-
-    auto redirect = [this, originalUrl, reply] {
-        const QString redir = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toString();
-        redirection(originalUrl.resolved(QUrl(redir)));
-    };
-
-    if (statusCode == 301) {
-        setMetaData(QStringLiteral("permanent-redirect"), QStringLiteral("true"));
-        redirect();
-    } else if (statusCode == 302) {
-        if (method == KIO::HTTP_POST) {
-            setMetaData(QStringLiteral("redirect-to-get"), QStringLiteral("true"));
-        }
-
-        redirect();
-    } else if (statusCode == 303) {
-        if (method != KIO::HTTP_HEAD) {
-            setMetaData(QStringLiteral("redirect-to-get"), QStringLiteral("true"));
-        }
-
-        redirect();
-    } else if (statusCode == 307) {
-        redirect();
-    } else if (statusCode == 308) {
-        setMetaData(QStringLiteral("permanent-redirect"), QStringLiteral("true"));
-        redirect();
-    }
-}
-
 KIO::WorkerResult HTTPProtocol::listDir(const QUrl &url)
 {
     return davStatList(url, false);
@@ -728,11 +777,6 @@ KIO::WorkerResult HTTPProtocol::davStatList(const QUrl &url, bool stat)
 
     Response response = makeDavRequest(url, method, inputData, DataMode::Return, extraHeaders);
 
-    // TODO
-    // if (!stat) {
-    // Utils::appendSlashToPath(m_request.url);
-    // }
-
     // Has a redirection already been called? If so, we're done.
     // if (m_isRedirection || m_kioError) {
     // if (m_isRedirection) {
@@ -1058,7 +1102,7 @@ int HTTPProtocol::codeFromResponse(const QString &response)
 
 KIO::WorkerResult HTTPProtocol::stat(const QUrl &url)
 {
-    if (url.scheme() != QLatin1String("webdav") && url.scheme() != QLatin1String("webdavs")) {
+    if (!isDav(url.scheme())) {
         QString statSide = metaData(QStringLiteral("statSide"));
         if (statSide != QLatin1String("source")) {
             // When uploading we assume the file does not exist.
@@ -1101,26 +1145,6 @@ KIO::WorkerResult HTTPProtocol::rename(const QUrl &src, const QUrl &dest, KIO::J
     QByteArray inputData;
     Response response = makeDavRequest(src, KIO::DAV_MOVE, inputData, DataMode::Discard, extraHeaders);
 
-    // Work around strict Apache-2 WebDAV implementation which refuses to cooperate
-    // with webdav://host/directory, instead requiring webdav://host/directory/
-    // (strangely enough it accepts Destination: without a trailing slash)
-    // See BR# 209508 and BR#187970
-    // TODO
-    // if (m_request.responseCode == 301) {
-    //     QUrl redir = m_request.redirectUrl;
-    //
-    //     resetSessionSettings();
-    //
-    //     m_request.url = redir;
-    //     m_request.method = DAV_MOVE;
-    //     m_request.davData.desturl = newDest.toString();
-    //     m_request.davData.overwrite = (flags & KIO::Overwrite);
-    //     m_request.url.setQuery(QString());
-    //     m_request.cacheTag.policy = CC_Reload;
-    //
-    //     (void)/* handling result via dav codes */ proceedUntilResponseHeader();
-    // }
-
     // The server returns a HTTP/1.1 201 Created or 204 No Content on successful completion
     if (response.httpCode == 201 || response.httpCode == 204) {
         return KIO::WorkerResult::pass();
@@ -1163,7 +1187,7 @@ KIO::WorkerResult HTTPProtocol::copy(const QUrl &src, const QUrl &dest, int, KIO
 
 KIO::WorkerResult HTTPProtocol::del(const QUrl &url, bool)
 {
-    if (url.scheme().startsWith(QLatin1String("webdav"))) {
+    if (isDav(url.scheme())) {
         Response response = makeRequest(url, KIO::HTTP_DELETE, {}, DataMode::Discard);
 
         // The server returns a HTTP/1.1 200 Ok or HTTP/1.1 204 No Content
@@ -1684,5 +1708,68 @@ QByteArray HTTPProtocol::methodToString(KIO::HTTP_METHOD method)
     }
 }
 
+// This is not the OS, but the windowing system, e.g. X11 on Unix/Linux.
+static QString platform()
+{
+#if defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN)
+    return QStringLiteral("X11");
+#elif defined(Q_OS_MAC)
+    return QStringLiteral("Macintosh");
+#elif defined(Q_OS_WIN)
+    return QStringLiteral("Windows");
+#else
+    return QStringLiteral("Unknown");
+#endif
+}
+
+QString HTTPProtocol::defaultUserAgent()
+{
+    if (!m_defaultUserAgent.isEmpty()) {
+        return m_defaultUserAgent;
+    }
+
+    QString systemName;
+    QString machine;
+    QString supp;
+    const bool sysInfoFound = getSystemNameVersionAndMachine(systemName, machine);
+
+    supp += platform();
+
+    if (sysInfoFound) {
+        supp += QLatin1String("; ") + systemName;
+        supp += QLatin1Char(' ') + machine;
+    }
+
+    QString appName = QCoreApplication::applicationName();
+    if (appName.isEmpty() || appName.startsWith(QLatin1String("kcmshell"), Qt::CaseInsensitive)) {
+        appName = QStringLiteral("KDE");
+    }
+    QString appVersion = QCoreApplication::applicationVersion();
+    if (appVersion.isEmpty()) {
+        appVersion += QLatin1String(KIO_VERSION_STRING);
+    }
+
+    m_defaultUserAgent = QLatin1String("Mozilla/5.0 (%1) ").arg(supp)
+        + QLatin1String("KIO/%1.%2 ").arg(QString::number(KIO_VERSION_MAJOR), QString::number(KIO_VERSION_MINOR))
+        + QLatin1String("%1/%2").arg(appName, appVersion);
+
+    return m_defaultUserAgent;
+}
+
+bool HTTPProtocol::getSystemNameVersionAndMachine(QString &systemName, QString &machine)
+{
+#if defined(Q_OS_WIN)
+    systemName = QStringLiteral("Windows");
+#else
+    struct utsname unameBuf;
+    if (0 != uname(&unameBuf)) {
+        return false;
+    }
+    systemName = QString::fromUtf8(unameBuf.sysname);
+    machine = QString::fromUtf8(unameBuf.machine);
+#endif
+    return true;
+}
+
 #include "http.moc"
 #include "moc_http.cpp"
