@@ -20,6 +20,9 @@
 #include <KDEDModule>
 #include <kio/authinfo.h>
 
+class KMessageDialog;
+class KPasswordDialog;
+
 namespace KWallet
 {
 class Wallet;
@@ -31,7 +34,7 @@ class KPasswdServer : public KDEDModule, protected QDBusContext
 
 public:
     explicit KPasswdServer(QObject *parent, const QList<QVariant> & = QList<QVariant>());
-    ~KPasswdServer();
+    ~KPasswdServer() override;
 
     // Called by the unit test
     void setWalletDisabled(bool d)
@@ -59,37 +62,38 @@ Q_SIGNALS:
     void queryAuthInfoAsyncResult(qlonglong requestId, qlonglong seqNr, const KIO::AuthInfo &);
 
 private Q_SLOTS:
-    void passwordDialogDone(int);
-    void retryDialogDone(int);
+    void passwordDialogDone(int result, KPasswordDialog *sender);
+    void retryDialogDone(int result, KMessageDialog *sender);
     void windowRemoved(WId);
 
 private:
     struct AuthInfoContainer {
         AuthInfoContainer()
-            : expire(expNever)
-            , seqNr(0)
-            , isCanceled(false)
         {
         }
 
         KIO::AuthInfo info;
         QString directory;
 
-        enum { expNever, expWindowClose, expTime } expire;
+        enum {
+            expNever,
+            expWindowClose,
+            expTime
+        } expire;
         QList<qlonglong> windowList;
-        qulonglong expireTime;
-        qlonglong seqNr;
+        qulonglong expireTime = expNever;
+        qlonglong seqNr = 0;
 
-        bool isCanceled;
+        bool isCanceled = false;
 
         struct Sorter {
-            bool operator()(AuthInfoContainer *n1, AuthInfoContainer *n2) const;
+            bool operator()(const AuthInfoContainer &n1, const AuthInfoContainer &n2) const;
         };
     };
 
     struct Request {
-        bool isAsync;             // true for async requests
-        qlonglong requestId;      // set for async requests only
+        bool isAsync; // true for async requests
+        qlonglong requestId; // set for async requests only
         QDBusMessage transaction; // set for sync requests only
         QString key;
         KIO::AuthInfo info;
@@ -106,7 +110,7 @@ private:
     void copyAuthInfo(const AuthInfoContainer *, KIO::AuthInfo &);
     void updateAuthExpire(const QString &key, const AuthInfoContainer *, qlonglong windowId, bool keep);
 
-#ifdef HAVE_KF5WALLET
+#ifdef HAVE_KF6WALLET
     bool openWallet(qlonglong windowId);
 #endif
 
@@ -115,7 +119,7 @@ private:
     void showPasswordDialog(Request *request);
     void updateCachedRequestKey(QList<Request *> &, const QString &oldKey, const QString &newKey);
 
-    typedef QList<AuthInfoContainer *> AuthInfoContainerList;
+    using AuthInfoContainerList = QList<AuthInfoContainer>;
     QHash<QString, AuthInfoContainerList *> m_authDict;
 
     QList<Request *> m_authPending;
