@@ -88,6 +88,7 @@ public:
         , m_mimeData(dropEvent->mimeData()) // Extract everything from the dropevent, since it will be deleted before the job starts
         , m_urls(KUrlMimeData::urlsFromMimeData(m_mimeData, KUrlMimeData::PreferLocalUrls, &m_metaData))
         , m_dropAction(dropEvent->dropAction())
+        , m_possibleActions(dropEvent->possibleActions())
         , m_relativePos(dropEvent->position().toPoint())
         , m_keyboardModifiers(dropEvent->modifiers())
         , m_hasArkFormat(m_mimeData->hasFormat(s_applicationSlashXDashKDEDashArkDashDnDExtractDashService)
@@ -303,8 +304,6 @@ void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
 {
     Q_Q(DropJob);
 
-    Q_EMIT q->popupMenuAboutToShow(m_itemProps);
-
     const int separatorLength = QCoreApplication::translate("QShortcut", "+").size();
     QString seq = QKeySequence(Qt::ShiftModifier).toString(QKeySequence::NativeText);
     seq.chop(separatorLength); // chop superfluous '+'
@@ -453,7 +452,12 @@ void DropJobPrivate::handleCopyToDirectory()
         if (mountPoints.isEmpty()) {
             mountPoints = KMountPoint::currentMountPoints();
         }
-        destDevice = mountPoints.findByPath(m_destUrl.path())->mountedFrom();
+        KMountPoint::Ptr destMountPoint = mountPoints.findByPath(m_destUrl.path());
+        if (destMountPoint) {
+            destDevice = destMountPoint->mountedFrom();
+        } else {
+            qCWarning(KIO_WIDGETS) << "Could not determine mount point for destination drop target " << m_destUrl;
+        }
     } else {
         allItemsAreSameDevice = false;
     }
@@ -499,10 +503,17 @@ void DropJobPrivate::handleCopyToDirectory()
             if (mountPoints.isEmpty()) {
                 mountPoints = KMountPoint::currentMountPoints();
             }
-            const QString &sourceDevice = mountPoints.findByPath(url.path())->mountedFrom();
+            QString sourceDevice;
+            KMountPoint::Ptr sourceMountPoint = mountPoints.findByPath(url.path());
+            if (sourceMountPoint) {
+                sourceDevice = sourceMountPoint->mountedFrom();
+            } else {
+                qCWarning(KIO_WIDGETS) << "Could not determine mount point for destination drag source " << url;
+            }
             if (sourceDevice != destDevice && !KFileItem(url).isLink()) {
                 allItemsAreSameDevice = false;
-            } else if (sourceDevice.isEmpty()) {
+            }
+            if (sourceDevice.isEmpty()) {
                 // Sanity check in case we somehow have a local files that we can't get the mount points from.
                 allItemsAreSameDevice = false;
             }
@@ -521,21 +532,19 @@ void DropJobPrivate::handleCopyToDirectory()
     }
     m_itemProps.setItems(fileItems);
 
-    m_possibleActions = Qt::LinkAction;
+    m_possibleActions |= Qt::LinkAction;
     const bool sReading = m_itemProps.supportsReading();
     // For http URLs, even though technically the protocol supports deleting,
     // this never makes sense for a drag operation.
     const bool sDeleting = m_allSourcesAreHttpUrls ? false : m_itemProps.supportsDeleting();
     const bool sMoving = m_itemProps.supportsMoving();
 
-    if (sReading) {
-        m_possibleActions |= Qt::CopyAction;
+    if (!sReading) {
+        m_possibleActions &= ~Qt::CopyAction;
     }
 
-    if (sMoving || (sReading && sDeleting)) {
-        if (!equalDestination) {
-            m_possibleActions |= Qt::MoveAction;
-        }
+    if (!(sMoving || (sReading && sDeleting)) || equalDestination) {
+        m_possibleActions &= ~Qt::MoveAction;
     }
 
     const bool trashing = m_destUrl.scheme() == QLatin1String("trash");
@@ -641,6 +650,7 @@ void DropJobPrivate::slotDropActionDetermined(int error)
         });
 
         fillPopupMenu(menu);
+        Q_EMIT q->popupMenuAboutToShow(m_itemProps);
         QObject::connect(menu, &QMenu::triggered, q, [this](QAction *action) {
             m_triggered = true;
             slotTriggered(action);
