@@ -69,11 +69,6 @@ public Q_SLOTS:
         cout << "*** itemsDeleted: " << endl;
         // TODO
     }
-    void itemsFilteredByMime(const KFileItemList &)
-    {
-        cout << "*** itemsFilteredByMime: " << endl;
-        // TODO
-    }
     void refreshItems(const QList<QPair<KFileItem, KFileItem>> &)
     {
         cout << "*** refreshItems: " << endl;
